'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _suite = require('mocha/lib/suite');

var _suite2 = _interopRequireDefault(_suite);

var _test = require('mocha/lib/test');

var _test2 = _interopRequireDefault(_test);

var _common = require('mocha/lib/interfaces/common');

var _common2 = _interopRequireDefault(_common);

var _bdd = require('mocha/lib/interfaces/bdd');

var _bdd2 = _interopRequireDefault(_bdd);

var _MockStore = require('./MockStore');

var _MockStore2 = _interopRequireDefault(_MockStore);

var _ResultStore = require('./ResultStore');

var _ResultStore2 = _interopRequireDefault(_ResultStore);

var _mochaHelpers = require('./mochaHelpers');

var _when = require('./when');

var _when2 = _interopRequireDefault(_when);

var _then = require('./then');

var _then2 = _interopRequireDefault(_then);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

exports.default = function (suite) {
    var suites = [suite];
    suite.on('pre-require', function (context, file, mocha) {
        var common = (0, _common2.default)(suite, context);

        // eslint-disable-next-line no-param-reassign
        context.run = mocha.options.delay && common.runWithSuite(suite);

        // eslint-disable-next-line no-param-reassign
        context.feature = function (title, reducer, path, fn) {
            var pathDefined = !!fn;

            var feature = _suite2.default.create(suites[0], title);
            feature.file = file;
            feature.initial = new _MockStore2.default(reducer, pathDefined ? path : undefined);
            feature.results = new _ResultStore2.default();

            suites.unshift(feature);
            (pathDefined ? fn : path).call(feature);
            suites.shift();

            return feature;
        };

        // eslint-disable-next-line no-param-reassign
        context.scenario = function (title, fn) {
            (0, _invariant2.default)(suites[0].initial, 'Scenario must be inside feature.');

            var scenario = _suite2.default.create(suites[0], title);
            scenario.file = file;
            scenario.initial = suites[0].initial;
            scenario.results = suites[0].results;

            suites.unshift(scenario);
            fn.call(scenario);
            suites.shift();

            return scenario;
        };

        // eslint-disable-next-line no-param-reassign
        context.given = function (title) {
            (0, _invariant2.default)(suites[0].initial, 'Given must be inside scenario');

            var test = void 0;
            if (!title) {
                test = new _test2.default('given initial state', function given() {
                    this.store = this.test.parent.initial;
                });
            } else {
                test = new _test2.default('given ' + title, function given() {
                    var _this = this;

                    (0, _mochaHelpers.executeCommandWithDependentTests)(this.test, function () {
                        _this.store = _this.test.parent.results.get(title);
                    });
                });
            }

            test.file = file;
            suites[0].addTest(test);
            return test;
        };

        // eslint-disable-next-line no-param-reassign
        context.when = function (title) {
            for (var _len = arguments.length, actions = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
                actions[_key - 1] = arguments[_key];
            }

            var test = new _test2.default('when ' + title, (0, _when2.default)(actions));
            test.file = file;
            suites[0].addTest(test);
            return test;
        };

        // eslint-disable-next-line no-param-reassign
        context.then = function (title) {
            for (var _len2 = arguments.length, params = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
                params[_key2 - 1] = arguments[_key2];
            }

            var test = new _test2.default('then ' + title, (0, _then2.default)(params));
            test.file = file;
            suites[0].addTest(test);
            return test;
        };

        // eslint-disable-next-line no-param-reassign
        context.thenP = function (title, selector) {
            for (var _len3 = arguments.length, params = Array(_len3 > 2 ? _len3 - 2 : 0), _key3 = 2; _key3 < _len3; _key3++) {
                params[_key3 - 2] = arguments[_key3];
            }

            var assertion = params[params.length - 1];
            var paramValueSets = params.slice(0, -1);
            return paramValueSets.map(function (paramValues) {
                return (0, _then2.default)([selector].concat(_toConsumableArray(paramValues), [function (result) {
                    return assertion.apply(undefined, [result].concat(_toConsumableArray(paramValues)));
                }]));
            }).map(function (testFn, index) {
                var test = new _test2.default('then ' + title + ' [' + index + ']', testFn);
                test.file = file;
                suites[0].addTest(test);
                return test;
            });
        };

        // eslint-disable-next-line no-param-reassign
        context.result = function (title) {
            suites[0].afterAll(function storeResult() {
                this.test.parent.results.set(title, this.store);
            });
        };
    });

    (0, _bdd2.default)(suite);
};